package com.agilecontrol.nea.core.excel.xls2html;

import com.agilecontrol.nea.core.excel.xls2html.HSSFHtmlHelper;
import com.agilecontrol.nea.core.excel.xls2html.HtmlHelper;
import com.agilecontrol.nea.core.excel.xls2html.XSSFHtmlHelper;
import com.agilecontrol.nea.util.Tools;
import java.io.BufferedReader;
import java.io.Closeable;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.Formatter;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.format.CellFormat;
import org.apache.poi.ss.format.CellFormatResult;
import org.apache.poi.ss.usermodel.BorderStyle;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.usermodel.CellType;
import org.apache.poi.ss.usermodel.Font;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.VerticalAlignment;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.ss.usermodel.WorkbookFactory;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;

public class ToHtml {

   private final Workbook wb;
   private final Appendable output;
   private boolean completeHTML;
   private Formatter out;
   private boolean gotBounds;
   private int firstColumn;
   private int endColumn;
   private HtmlHelper helper;
   private static final String DEFAULTS_CLASS = "excelDefaults";
   private static final String COL_HEAD_CLASS = "colHeader";
   private static final String ROW_HEAD_CLASS = "rowHeader";
   private static final Map ALIGN = mapFor(new Object[]{HorizontalAlignment.LEFT, "left", HorizontalAlignment.CENTER, "center", HorizontalAlignment.RIGHT, "right"});
   private static final Map VERTICAL_ALIGN = mapFor(new Object[]{VerticalAlignment.BOTTOM, "bottom", VerticalAlignment.CENTER, "middle", VerticalAlignment.TOP, "top"});
   private static final Map BORDER = mapFor(new Object[]{BorderStyle.DASH_DOT, "solid 1pt", BorderStyle.DASH_DOT_DOT, "solid 1pt", BorderStyle.DASHED, "solid 1pt", BorderStyle.DOTTED, "solid 1pt", BorderStyle.DOUBLE, "double 3pt", BorderStyle.HAIR, "solid 1px", BorderStyle.MEDIUM, "solid 2pt", BorderStyle.MEDIUM_DASH_DOT, "dashed 2pt", BorderStyle.MEDIUM_DASH_DOT_DOT, "solid 2pt", BorderStyle.MEDIUM_DASHED, "solid 2pt", BorderStyle.NONE, "none", BorderStyle.SLANTED_DASH_DOT, "solid 2pt", BorderStyle.THICK, "solid 3pt", BorderStyle.THIN, "solid 1pt"});


   private static Map mapFor(Object ... mapping) {
      HashMap map = new HashMap();

      for(int i = 0; i < mapping.length; i += 2) {
         map.put(mapping[i], mapping[i + 1]);
      }

      return map;
   }

   private static ToHtml create(Workbook wb, Appendable output) {
      return new ToHtml(wb, output);
   }

   private static ToHtml create(String path, Appendable output) throws IOException {
      return create((InputStream)(new FileInputStream(path)), output);
   }

   private static ToHtml create(InputStream in, Appendable output) throws IOException {
      try {
         Workbook e = WorkbookFactory.create(in);
         return create((Workbook)e, output);
      } catch (Exception var3) {
         throw new IllegalArgumentException("Cannot create workbook from stream", var3);
      }
   }

   private ToHtml(Workbook wb, Appendable output) {
      if(wb == null) {
         throw new NullPointerException("wb");
      } else if(output == null) {
         throw new NullPointerException("output");
      } else {
         this.wb = wb;
         this.output = output;
         this.setupColorMap();
      }
   }

   private void setupColorMap() {
      if(this.wb instanceof HSSFWorkbook) {
         this.helper = new HSSFHtmlHelper((HSSFWorkbook)this.wb);
      } else {
         if(!(this.wb instanceof XSSFWorkbook)) {
            throw new IllegalArgumentException("unknown workbook type: " + this.wb.getClass().getSimpleName());
         }

         this.helper = new XSSFHtmlHelper();
      }

   }

   public static void main(String[] args) throws Exception {
      args = new String[]{"/share/dowloads/a.xls", "/share/dowloads/b.html"};
      if(args.length < 2) {
         System.err.println("usage: ToHtml inputWorkbook outputHtmlFile");
      } else {
         Tools.writeFile(args[1], build(args[0], true), "UTF-8");
      }
   }

   public static String build(String xlsFile, boolean isCompleteHTML) throws Exception {
      StringWriter sw = new StringWriter();
      ToHtml toHtml = create((String)xlsFile, new PrintWriter(sw));
      toHtml.setCompleteHTML(isCompleteHTML);
      toHtml.printPage();
      return sw.toString();
   }

   void setCompleteHTML(boolean completeHTML) {
      this.completeHTML = completeHTML;
   }

   void printPage() throws IOException {
      boolean var5 = false;

      try {
         var5 = true;
         this.ensureOut();
         if(this.completeHTML) {
            this.out.format("<?xml version=\"1.0\" encoding=\"UTF-8\" ?>%n", new Object[0]);
            this.out.format("<html>%n", new Object[0]);
            this.out.format("<head>%n", new Object[0]);
            this.out.format("</head>%n", new Object[0]);
            this.out.format("<body>%n", new Object[0]);
         }

         this.print();
         if(this.completeHTML) {
            this.out.format("</body>%n", new Object[0]);
            this.out.format("</html>%n", new Object[0]);
            var5 = false;
         } else {
            var5 = false;
         }
      } finally {
         if(var5) {
            if(this.out != null) {
               this.out.close();
            }

            if(this.output instanceof Closeable) {
               Closeable closeable1 = (Closeable)this.output;
               closeable1.close();
            }

         }
      }

      if(this.out != null) {
         this.out.close();
      }

      if(this.output instanceof Closeable) {
         Closeable closeable = (Closeable)this.output;
         closeable.close();
      }

   }

   void print() {
      this.printInlineStyle();
      this.printSheets();
   }

   private void printInlineStyle() {
      this.out.format("<style type=\"text/css\">%n", new Object[0]);
      this.printStyles();
      this.out.format("</style>%n", new Object[0]);
   }

   private void ensureOut() {
      if(this.out == null) {
         this.out = new Formatter(this.output);
      }

   }

   void printStyles() {
      this.ensureOut();
      BufferedReader in = null;

      try {
         in = new BufferedReader(new InputStreamReader(this.getClass().getResourceAsStream("excelStyle.css")));

         String seen;
         while((seen = in.readLine()) != null) {
            this.out.format("%s%n", new Object[]{seen});
         }
      } catch (IOException var15) {
         throw new IllegalStateException("Reading standard css", var15);
      } finally {
         if(in != null) {
            try {
               in.close();
            } catch (IOException var14) {
               throw new IllegalStateException("Reading standard css", var14);
            }
         }

      }

      HashSet var17 = new HashSet();

      for(int i = 0; i < this.wb.getNumberOfSheets(); ++i) {
         Sheet sheet = this.wb.getSheetAt(i);
         Iterator rows = sheet.rowIterator();

         while(rows.hasNext()) {
            Row row = (Row)rows.next();
            Iterator var7 = row.iterator();

            while(var7.hasNext()) {
               Cell cell = (Cell)var7.next();
               CellStyle style = cell.getCellStyle();
               if(!var17.contains(style)) {
                  this.printStyle(style);
                  var17.add(style);
               }
            }
         }
      }

   }

   private void printStyle(CellStyle style) {
      this.out.format(".%s .%s {%n", new Object[]{"excelDefaults", this.styleName(style)});
      this.styleContents(style);
      this.out.format("}%n", new Object[0]);
   }

   private void styleContents(CellStyle style) {
      this.styleOut("text-align", style.getAlignment(), ALIGN);
      this.styleOut("vertical-align", style.getVerticalAlignment(), VERTICAL_ALIGN);
      this.fontStyle(style);
      this.borderStyles(style);
      this.helper.colorStyles(style, this.out);
   }

   private void borderStyles(CellStyle style) {
      this.styleOut("border-left", style.getBorderLeft(), BORDER);
      this.styleOut("border-right", style.getBorderRight(), BORDER);
      this.styleOut("border-top", style.getBorderTop(), BORDER);
      this.styleOut("border-bottom", style.getBorderBottom(), BORDER);
   }

   private void fontStyle(CellStyle style) {
      Font font = this.wb.getFontAt(style.getFontIndex());
      if(font.getBold()) {
         this.out.format("  font-weight: bold;%n", new Object[0]);
      }

      if(font.getItalic()) {
         this.out.format("  font-style: italic;%n", new Object[0]);
      }

      short fontheight = font.getFontHeightInPoints();
      if(fontheight == 9) {
         fontheight = 10;
      }

      this.out.format("  font-size: %dpt;%n", new Object[]{Integer.valueOf(fontheight)});
   }

   private String styleName(CellStyle style) {
      if(style == null) {
         style = this.wb.getCellStyleAt(0);
      }

      StringBuilder sb = new StringBuilder();
      Formatter fmt = new Formatter(sb);

      String var4;
      try {
         fmt.format("style_%02x", new Object[]{Short.valueOf(style.getIndex())});
         var4 = fmt.toString();
      } finally {
         fmt.close();
      }

      return var4;
   }

   private void styleOut(String attr, Object key, Map mapping) {
      String value = (String)mapping.get(key);
      if(value != null) {
         this.out.format("  %s: %s;%n", new Object[]{attr, value});
      }

   }

   private static CellType ultimateCellType(Cell c) {
      CellType type = c.getCellType();
      if(type == CellType.FORMULA) {
         type = c.getCachedFormulaResultType();
      }

      return type;
   }

   private void printSheets() {
      this.ensureOut();
      Sheet sheet = this.wb.getSheetAt(0);
      this.printSheet(sheet);
   }

   void printSheet(Sheet sheet) {
      this.ensureOut();
      this.out.format("<table class=%s>%n", new Object[]{"excelDefaults"});
      this.printCols(sheet);
      this.printSheetContent(sheet);
      this.out.format("</table>%n", new Object[0]);
   }

   private void printCols(Sheet sheet) {
      this.out.format("<col/>%n", new Object[0]);
      this.ensureColumnBounds(sheet);

      for(int i = this.firstColumn; i < this.endColumn; ++i) {
         this.out.format("<col/>%n", new Object[0]);
      }

   }

   private void ensureColumnBounds(Sheet sheet) {
      if(!this.gotBounds) {
         Iterator iter = sheet.rowIterator();
         this.firstColumn = iter.hasNext()?Integer.MAX_VALUE:0;
         this.endColumn = 0;

         while(iter.hasNext()) {
            Row row = (Row)iter.next();
            short firstCell = row.getFirstCellNum();
            if(firstCell >= 0) {
               this.firstColumn = Math.min(this.firstColumn, firstCell);
               this.endColumn = Math.max(this.endColumn, row.getLastCellNum());
            }
         }

         this.gotBounds = true;
      }
   }

   private void printColumnHeads() {
      this.out.format("<thead>%n", new Object[0]);
      this.out.format("  <tr class=%s>%n", new Object[]{"colHeader"});
      this.out.format("    <th class=%s>L</th>%n", new Object[]{"colHeader"});
      StringBuilder colName = new StringBuilder();

      for(int i = this.firstColumn; i < this.endColumn; ++i) {
         colName.setLength(0);
         int cnum = i;

         do {
            colName.insert(0, (char)(65 + cnum % 26));
            cnum /= 26;
         } while(cnum > 0);

         this.out.format("    <th class=%s>%s</th>%n", new Object[]{"colHeader", colName});
      }

      this.out.format("  </tr>%n", new Object[0]);
      this.out.format("</thead>%n", new Object[0]);
   }

   private void printSheetContent(Sheet sheet) {
      this.printColumnHeads();
      this.out.format("<tbody>%n", new Object[0]);
      Iterator rows = sheet.rowIterator();

      while(rows.hasNext()) {
         Row row = (Row)rows.next();
         this.out.format("  <tr>%n", new Object[0]);
         this.out.format("    <td class=%s>%d</td>%n", new Object[]{"rowHeader", Integer.valueOf(row.getRowNum() + 1)});

         for(int i = this.firstColumn; i < this.endColumn; ++i) {
            String content = "&nbsp;";
            String attrs = "";
            CellStyle style = null;
            if(i >= row.getFirstCellNum() && i < row.getLastCellNum()) {
               Cell cell = row.getCell(i);
               if(cell != null) {
                  style = cell.getCellStyle();
                  attrs = this.tagStyle(cell, style);
                  CellFormat cf = CellFormat.getInstance(style.getDataFormatString());
                  CellFormatResult result = cf.apply(cell);
                  content = result.text;
                  if(content.equals("")) {
                     content = "&nbsp;";
                  }
               }
            }

            this.out.format("    <td class=%s %s>%s</td>%n", new Object[]{this.styleName(style), attrs, content});
         }

         this.out.format("  </tr>%n", new Object[0]);
      }

      this.out.format("</tbody>%n", new Object[0]);
   }

   private String tagStyle(Cell cell, CellStyle style) {
      if(style.getAlignment().equals(HorizontalAlignment.CENTER)) {
         switch(null.$SwitchMap$org$apache$poi$ss$usermodel$CellType[ultimateCellType(cell).ordinal()]) {
         case 1:
            return "style=\"text-align: left;\"";
         case 2:
         case 3:
            return "style=\"text-align: center;\"";
         case 4:
         }
      }

      return "";
   }

}
